@@ -1,5 +1,6 @@
 exams2canvas <- function(file, n = 1L, dir = ".", name = "canvasquiz",
-  maxattempts = 1, duration = NULL, points = NULL, converter = NULL, ...)
+  maxattempts = 1, duration = NULL, points = NULL, converter = NULL,
+  template = "canvas_qti12.xml", ...)
 {
   ## enforce MathML for mathematical notation
   if(any(tolower(tools::file_ext(unlist(file))) == "rmd")) {
@@ -20,7 +21,8 @@ exams2canvas <- function(file, n = 1L, dir = ".", name = "canvasquiz",
   rval <- exams2qti12(file = file, n = n, dir = dir, name = name,
     maxattempts = maxattempts, duration = duration, points = points,
     converter = converter, flavor = "canvas", base64 = FALSE,
-    eval = list(partial = TRUE, negative = FALSE), ...)
+    eval = list(partial = TRUE, negative = FALSE),
+    template = template, ...)
 
   invisible(rval)
 }
