@@ -1,12 +1,13 @@
 ## generate exams in Moodle XML format
 ## http://docs.moodle.org/en/Moodle_XML_format
 exams2moodle <- function(file, n = 1L, nsamp = NULL, dir = ".",
-  name = NULL, quiet = TRUE, edir = NULL, tdir = NULL, sdir = NULL, verbose = FALSE,
-  resolution = 100, width = 4, height = 4, svg = FALSE, encoding = "", 
+  name = NULL, quiet = TRUE, edir = NULL, tdir = NULL, sdir = NULL, verbose = FALSE, rds = FALSE,
+  resolution = 100, width = 4, height = 4, svg = FALSE, encoding = "UTF-8", 
   iname = TRUE, stitle = NULL, testid = FALSE, zip = FALSE,
   num = NULL, mchoice = NULL, schoice = mchoice, string = NULL, cloze = NULL,
-  points = NULL, rule = NULL, pluginfile = TRUE,
-  converter = "pandoc-mathjax", envir = NULL, ...)
+  points = NULL, rule = NULL, pluginfile = TRUE, forcedownload = FALSE,
+  converter = "pandoc-mathjax", envir = NULL,
+  table = NULL, css = NULL, ...)
 {
   ## default converter is "ttm" if all exercises are Rnw, otherwise "pandoc"
   if(is.null(converter)) {
@@ -15,15 +16,35 @@ exams2moodle <- function(file, n = 1L, nsamp = NULL, dir = ".",
   ## set up .html transformer
   htmltransform <- make_exercise_transform_html(converter = converter, ..., base64 = !pluginfile)
 
+  ## encoding always assumed to be UTF-8 starting from R/exams 2.4-0
+  if(!is.null(encoding) && !(tolower(encoding) %in% c("", "utf-8", "utf8"))) {
+    warning("the only supported 'encoding' is UTF-8")
+  }
+  encoding <- "UTF-8"
+
+  ## change <table> class for custom CSS in Moodle
+  if(!is.null(table)) {
+    if(isTRUE(table)) table <- "table_shade"
+    .exams_set_internal(pandoc_table_class_fixup = table)
+    on.exit(.exams_set_internal(pandoc_table_class_fixup = FALSE))
+
+    if(is.null(css) && table %in% c("table_grid", "table_rule", "table_shade")) {
+      css <- readLines(system.file(file.path("css", "table.css"), package = "exams"))
+    }
+  }
+
+  ## create a name
+  if(is.null(name)) name <- "moodlequiz"
+  if(isTRUE(rds)) rds <- name
+
   ## generate the exam
-  if(encoding == "") encoding <- "UTF-8"
   exm <- xexams(file, n = n, nsamp = nsamp,
    driver = list(
        sweave = list(quiet = quiet, pdf = FALSE, png = !svg, svg = svg,
          resolution = resolution, width = width, height = height,
          encoding = encoding, envir = envir),
        read = NULL, transform = htmltransform, write = NULL),
-     dir = dir, edir = edir, tdir = tdir, sdir = sdir, verbose = verbose)
+     dir = dir, edir = edir, tdir = tdir, sdir = sdir, verbose = verbose, rds = rds, points = points)
 
   ## get the possible moodle question body functions and options
   moodlequestion = list(num = num, mchoice = mchoice, schoice = schoice, cloze = cloze, string = string)
@@ -35,9 +56,8 @@ exams2moodle <- function(file, n = 1L, nsamp = NULL, dir = ".",
         moodlequestion[[i]]$eval <- list("partial" = TRUE, "rule" = rule)
       if(is.list(moodlequestion[[i]]$eval)) {
         if(!moodlequestion[[i]]$eval$partial) stop("Moodle can only process partial credits!")
-        if(i == "cloze" & is.null(moodlequestion[[i]]$eval$rule))
-          moodlequestion[[i]]$eval$rule <- "none"
       }
+      if(is.null(moodlequestion[[i]]$css)) moodlequestion[[i]]$css <- css
       moodlequestion[[i]] <- do.call("make_question_moodle", moodlequestion[[i]])
     }
     if(!is.function(moodlequestion[[i]])) stop(sprintf("wrong specification of %s", sQuote(i)))
@@ -57,9 +77,6 @@ exams2moodle <- function(file, n = 1L, nsamp = NULL, dir = ".",
   nx <- length(exm)
   nq <- length(exm[[1L]])
 
-  ## create a name
-  if(is.null(name)) name <- "moodlequiz"
-
   ## function for internal ids
   make_test_ids <- function(n, type = c("test", "section", "item"))
   {
@@ -73,7 +90,7 @@ exams2moodle <- function(file, n = 1L, nsamp = NULL, dir = ".",
   test_id <- make_test_ids(type = "test")
 
   ## create the directory where the test is stored
-  dir.create(test_dir <- file.path(tdir, name))
+  dir.create(test_dir <- file.path(file_path_as_absolute(tdir), name))
 
   exsecs <- rep(NA, length = nq)
   if(!is.null(stitle)) {
@@ -81,11 +98,7 @@ exams2moodle <- function(file, n = 1L, nsamp = NULL, dir = ".",
     exsecs[1:ks] <- stitle
   }
 
-  ## points setting
-  if(!is.null(points))
-    points <- rep(points, length.out = nq)
-
-  ## encoding
+  ## encoding (actually only UTF-8 supported starting from 2.4-0)
   enc <- gsub("-", "", tolower(encoding), fixed = TRUE)
   if(enc %in% c("iso8859", "iso88591")) enc <- "latin1"
   if(enc == "iso885915") enc <- "latin9"
@@ -123,9 +136,6 @@ exams2moodle <- function(file, n = 1L, nsamp = NULL, dir = ".",
 
     ## create the questions
     for(i in 1:nx) {
-      ## overule points
-      if(!is.null(points)) exm[[i]][[j]]$metainfo$points <- points[[j]]
-
       ## get the question type
       type <- exm[[i]][[j]]$metainfo$type
 
@@ -152,9 +162,9 @@ exams2moodle <- function(file, n = 1L, nsamp = NULL, dir = ".",
         for(si in seq_along(exm[[i]][[j]]$supplements)) {
 	  f <- basename(exm[[i]][[j]]$supplements[si])
 	  href <- paste0("\"", f,"\"")
-          if(any(grepl(href, question_xml))) {
+          if(any(grepl(href, question_xml, fixed = TRUE))) {
             if(isTRUE(pluginfile)) {
-              newfn   <- paste0("@@PLUGINFILE@@/", f)
+              newfn   <- paste0("@@PLUGINFILE@@/", f, if(isTRUE(forcedownload)) "?forcedownload=1" else "")
               newhref <- paste0("\"", newfn,"\"")
               filetag <- paste0("<file name=\"", f, "\" encoding=\"base64\">",
                                 base64enc::base64encode(exm[[i]][[j]]$supplements[si]),
@@ -165,10 +175,11 @@ exams2moodle <- function(file, n = 1L, nsamp = NULL, dir = ".",
 
               # Insert base64 encoded file at the end of <questiontext>
               idx <- which(grepl(newhref, question_xml, fixed = TRUE))
-              textend <- which(grepl("</text>", question_xml, fixed = TRUE))
-              textend <- head(textend[textend > idx], 1)
-
-              question_xml <- append(question_xml, filetag, after = textend)
+              for(k in idx) {
+                textend <- which(grepl("</text>", question_xml, fixed = TRUE))
+                textend <- head(textend[textend > k], 1)
+                question_xml <- append(question_xml, filetag, after = textend)
+              }
             } else {
               question_xml <- gsub(href,
                 paste0('"', fileURI(exm[[i]][[j]]$supplements[si]), '"'),
@@ -210,23 +221,15 @@ exams2moodle <- function(file, n = 1L, nsamp = NULL, dir = ".",
 ## Moodle question constructor function (originally for Moodle 2.3)
 make_question_moodle <-
 make_question_moodle23 <- function(name = NULL, solution = TRUE, shuffle = FALSE, penalty = 0,
-  answernumbering = "abc", usecase = FALSE, cloze_mchoice_display = "MULTICHOICE",
+  answernumbering = "abc", usecase = FALSE, cloze_mchoice_display = NULL, cloze_schoice_display = NULL,
   truefalse = c("True", "False"), enumerate = TRUE, abstention = NULL,
   eval = list(partial = TRUE, negative = FALSE, rule = "false2"),
-  essay = NULL)
-
+  essay = NULL, numwidth = NULL, stringwidth = NULL, css = NULL)
 {
   function(x) {
     ## how many points?
     points <- if(is.null(x$metainfo$points)) 1 else x$metainfo$points
 
-    ## choice policy
-    eval <- if(!all(names(exams_eval()) %in% names(eval))) {
-      if(x$metainfo$type == "cloze" & is.null(eval$rule))
-        eval$rule <- "none"
-      do.call("exams_eval", eval)
-    } else eval
-
     ## match question type
     type <- switch(x$metainfo$type,
       "num" = "numerical",
@@ -236,9 +239,24 @@ make_question_moodle23 <- function(name = NULL, solution = TRUE, shuffle = FALSE
       "string" = "shortanswer"
     )
 
+    if(type == "cloze") {
+      if(length(x$metainfo$clozetype) < 3L) {
+        if(all(x$metainfo$clozetype %in% c("essay", "file"))) {
+          x$metainfo$stringtype <- x$metainfo$clozetype
+          type <- "shortanswer"
+        }
+      }
+    }
+
     if(type == "shortanswer" && (isTRUE(x$metainfo$essay) || isTRUE(essay))) {
         type <- "essay"
     }
+    if(type == "shortanswer") {
+      if(!is.null(x$metainfo$stringtype)) {
+        if(any(grepl("essay", x$metainfo$stringtype)) | any(grepl("file", x$metainfo$stringtype)))
+          type <- "essay"
+      }
+    }
 
     ## question name
     if(is.null(name)) name <- x$metainfo$name
@@ -248,6 +266,19 @@ make_question_moodle23 <- function(name = NULL, solution = TRUE, shuffle = FALSE
     if(is.null(abstention) || identical(abstention, FALSE)) abstention <- ""
     if(isTRUE(abstention)) abstention <- "Abstention"
 
+    ## read CSS files
+    if(!is.null(css) && all(tools::file_ext(css) == "css")) {
+      css_inst <- system.file(file.path("css", css), package = "exams")
+      css <- ifelse(!file.exists(css) & file.exists(css_inst), css_inst, css)
+      if(!all(file.exists(css))) stop(paste("The following CSS files cannot be found: ",
+        paste(css[!file.exists(css)], collapse = ", "), ".", sep = ""))
+      css <- do.call("cbind", lapply(css, readLines))
+    }
+    ## turn into <style> tag
+    if(!is.null(css) && !grepl("<style", css[1L], fixed = TRUE)) {
+      css <- c('<style type="text/css" rel="stylesheet">', css, '</style>')
+    }
+
     ## start the question xml
     xml <- c(
       paste('\n<question type="', type, '">', sep = ''),
@@ -255,7 +286,11 @@ make_question_moodle23 <- function(name = NULL, solution = TRUE, shuffle = FALSE
       paste('<text>', name, '</text>'),
       '</name>',
       '<questiontext format="html">',
-      '<text><![CDATA[<p>', if(type != "cloze") x$question else '##QuestionText##', '</p>]]></text>',
+      '<text><![CDATA[',
+      css,
+      '<p>',
+      if(type != "cloze") x$question else '##QuestionText##',
+      '</p>]]></text>',
       '</questiontext>'
     )
 
@@ -277,8 +312,13 @@ make_question_moodle23 <- function(name = NULL, solution = TRUE, shuffle = FALSE
     }
 
     ## penalty and points
-    if(type == "cloze")
-      points <- rep(points, length.out = length(x$metainfo$solution))
+    if((length(points) == 1) & (type == "cloze")) {
+      #points <- points / length(x$metainfo$solution)
+      points <- rep(points, length = length(x$metainfo$solution))
+    }
+
+    points2 <- points * as.integer(paste0(c(1, rep(0, max(sapply(points, dc)))), collapse = ""))
+
     xml <- c(xml,
       paste('<penalty>', penalty, '</penalty>', sep = ''),
       paste('<defaultgrade>', sum(points), '</defaultgrade>', sep = '')
@@ -292,6 +332,8 @@ make_question_moodle23 <- function(name = NULL, solution = TRUE, shuffle = FALSE
         paste('<answernumbering>', answernumbering, '</answernumbering>', sep = '')
       )
 
+      ## evaluation policy
+      if(!("pointvec" %in% names(eval))) eval <- do.call("exams_eval", eval)
       frac <- as.integer(x$metainfo$solution)
       pv <- eval$pointvec(paste(frac, sep = "", collapse = ""))
       pv[pv == -Inf] <- 0 ## FIXME: exams_eval() return -Inf when rule = "none"?
@@ -351,37 +393,76 @@ make_question_moodle23 <- function(name = NULL, solution = TRUE, shuffle = FALSE
         essay_opts <- list(format="plain", required=TRUE, fieldlines=5L,
             attachments=0L, attachmentsrequired=FALSE)
 
+        if(!is.null(x$metainfo$stringtype)) {
+          if(any(grepl("file", x$metainfo$stringtype))) {
+            essay_opts$attachments <- 1L
+            if(length(x$metainfo$stringtype) == 1L)
+              essay_opts$fieldlines <- 0L
+          }
+        }
+
         if(!is.list(essay)) {
-            essay <- list()
+          essay <- list()
         }
 
         for(i in names(essay_opts)) {
-            vn <- paste0("essay_", i)
-            value <- x$metainfo[[vn]]
+          vn <- paste0("essay_", i)
+          value <- x$metainfo[[vn]]
+          if(!is.null(value)) {
+            essay_opts[[i]] <- value
+          }
+        }
 
-            if(!is.null(value)) {
-                essay_opts[[i]] <- value
-            }
+        i <- grep("essay_", names(x$metainfo), fixed = TRUE, value = TRUE)
+        if(length(i)) {
+          for(j in i) {
+            jn <- gsub("essay_", "", j, fixed = TRUE)
+            essay_opts[[jn]] <- x$metainfo[[j]]
+          }
         }
 
         for(i in names(essay_opts)) {
-            if(!is.null(essay[[i]])) essay_opts[[i]] <- essay[[i]]
+          if(!is.null(essay[[i]])) essay_opts[[i]] <- essay[[i]]
+        }
+
+        if((essay_opts$fieldlines < 1L) | !essay_opts$required) {
+          essay_opts$required <- FALSE
+          essay_opts$format <- "noinline"
+          essay_opts$attachmentsrequired <- TRUE
+          if(essay_opts$attachments < 1L) {
+            essay_opts$attachments <- 1L
+          }
         }
 
         txt <- paste0(
-            "<responseformat>", essay_opts$format, "</responseformat>\n",
-            "<responserequired>", as.integer(essay_opts$required), "</responserequired>\n",
-            "<responsefieldlines>", essay_opts$fieldlines, "</responsefieldlines>\n",
-            "<attachments>", essay_opts$attachments, "</attachments>\n",
-            "<attachmentsrequired>", as.integer(essay_opts$attachmentsrequired), "</attachmentsrequired>\n"
+          "<responseformat>", essay_opts$format, "</responseformat>\n",
+          "<responserequired>", as.integer(essay_opts$required), "</responserequired>\n",
+          "<responsefieldlines>", essay_opts$fieldlines, "</responsefieldlines>\n",
+          "<attachments>", essay_opts$attachments, "</attachments>\n",
+          "<attachmentsrequired>", as.integer(essay_opts$attachmentsrequired), "</attachmentsrequired>\n"
         )
 
+        if(!is.null(essay_opts$wordlimit)) {
+          if(length(essay_opts$wordlimit) < 2) {
+            txt <- c(txt, paste0('<minwordlimit></minwordlimit>\n<maxwordlimit>',
+              essay_opts$wordlimit[1L], '</maxwordlimit>'))
+          } else {
+            txt <- c(txt, paste0('<minwordlimit>', essay_opts$wordlimit[1L],
+              '</minwordlimit>\n<maxwordlimit>', essay_opts$wordlimit[2L], '</maxwordlimit>'))
+          }
+        }
+
         xml <- c(xml, txt)
     }
 
 
     ## cloze type questions
     if(type == "cloze") {
+      if(!all(points2 == points)) {
+        if(any((points %% 1) > 0))
+          warning("non-integer points, please check points in Moodle")
+      }
+
       ## how many questions
       solution <- if(!is.list(x$metainfo$solution)) {
         list(x$metainfo$solution)
@@ -407,40 +488,54 @@ make_question_moodle23 <- function(name = NULL, solution = TRUE, shuffle = FALSE
 
       ## optionally fix the num answer field width
       ## by supplying an additional wrong answer
-      numwidth <- if(is.null(x$metainfo$numwidth)) FALSE else TRUE
-      if(numwidth) {
-        nums <- NULL
-        for(i in 1:n) {
-          ql <- if(is.null(questionlist)) "" else questionlist[sid == i]
-          k <- length(ql)
-          if(x$metainfo$clozetype[i] == "num") {
-            for(j in 1:k) {
-              nums <- rbind(nums,
-                c(solution[[i]][j] - max(tol[[i]]),
-                solution[[i]][j] + max(tol[[i]])))
-            }
-          }
-        }
-        if(!is.null(nums)) {
-          if(is.logical(x$metainfo$numwidth)) {
-            fnums <- format(as.numeric(nums))
-          } else {
-            fnums <- if(!is.character(x$metainfo$numwidth)) {
-              paste(rep("1", length = as.integer(x$metainfo$numwidth)), sep = "", collapse = "")
-            } else x$metainfo$numwidth
-          }
-          num_w <- max(unlist(sapply(fnums, nchar)))
-          do <- TRUE
-          while(do) {
-            fnums <- make_id(num_w)
-            tolcheck <- NULL
-            for(i in 1:nrow(nums)) {
-              tolcheck <- c(tolcheck, fnums >= nums[i, 1] & fnums <= nums[i, 2])
-            }
-            do <- (fnums %in% nums) & any(tolcheck)
-          }
+      if(is.null(numwidth)) numwidth <- x$metainfo$numwidth
+      if(is.null(numwidth)) numwidth <- FALSE
+      numcloze <- x$metainfo$clozetype == "num"
+      if(!identical(numwidth, FALSE) && any(numcloze)) {
+        ## all correct numeric solutions
+        nums <- unlist(x$metainfo$solution[numcloze])
+	## +/- corresponding tolerance
+	nums <- cbind(nums - unlist(tol[numcloze]), nums + unlist(tol[numcloze]))
+
+        ## formatted number (of a wrong solution to enforce the width)
+        fnum <- if(is.logical(numwidth)) {
+          gsub(" ", "", format(as.numeric(nums)), fixed = TRUE)
+        } else if(is.character(numwidth)) {
+	  numwidth
+	} else {
+          paste(rep.int("9", as.integer(numwidth)), collapse = "")
         }
+	
+	## make sure that the formatted number is not within tolerance of any correct solution
+        fnum <- fnum[which.max(nchar(fnum))]
+	num_w <- nchar(fnum)
+	while(any(as.numeric(fnum) == nums) || any(as.numeric(fnum) >= nums[, 1] & as.numeric(fnum) <= nums[, 2])) {
+	  fnum <- make_id(num_w)
+	}
       }
+      
+      ## analogously fix the string width
+      ## by supplying an additional wrong answer
+      if(is.null(stringwidth)) stringwidth <- x$metainfo$stringwidth
+      if(is.null(stringwidth)) stringwidth <- FALSE
+      stringcloze <- x$metainfo$clozetype == "string"
+      if(!identical(stringwidth, FALSE) && any(stringcloze)) {
+        strings <- unlist(x$metainfo$solution[stringcloze])
+	fstring <- if(is.logical(stringwidth)) {
+	  strings[which.max(nchar(strings))]
+	} else if(is.character(stringwidth)) {
+	  stringwidth
+	} else {
+	  paste(rep.int("9", as.integer(stringwidth)), collapse = "")
+	}	
+	string_w <- nchar(fstring)
+	while(any(tolower(fstring) == tolower(strings))) {
+	  fstring <- paste(sample(base::LETTERS, string_w, replace = TRUE), collapse = "")
+	}
+      }
+
+      if(any(x$metainfo$clozetype %in% c("essay", "file")))
+        stop("essays or file uploads are not currently supported in Moodle cloze type exercises!")
 
       ## cycle through all questions
       qtext <- NULL; inum <- 1
@@ -449,48 +544,99 @@ make_question_moodle23 <- function(name = NULL, solution = TRUE, shuffle = FALSE
         k <- length(ql)
         tmp <- NULL
         if(grepl("choice", x$metainfo$clozetype[i])) {
-          tmp <- paste('{', points[i], ':', cloze_mchoice_display, ':', sep = '')
+          if(any(grepl("}", ql, fixed = TRUE))) {
+            ## due to "{1:TYPE:...}" markup in cloze questions it is necessary
+            ## to escape closing curly brackets in the questionlist
+            ql <- gsub("}", "\\}", ql, fixed = TRUE)
+          }
 
+          ## Moodle multiple-choice and single-choice displays
+          moodle_schoice_display <- c("MULTICHOICE", "MC", "MULTICHOICE_V", "MCV", "MULTICHOICE_H", "MCH",
+	    "MULTICHOICE_S", "MCS", "MULTICHOICE_VS", "MCVS", "MULTICHOICE_HS", "MCHS")
+          moodle_mchoice_display <- c("MULTIRESPONSE", "MR", "MULTIRESPONSE_H", "MRH", "MULTIRESPONSE_S", "MRS", "MULTIRESPONSE_HS", "MRHS")
+          ## select display type, defaults:
+	  ## - MULTIRESPONSE for mchoice items
+	  ## - MULTICHOICE_V for schoice items with math markup \(...\) which isn't supported in drop-down menus
+	  ## - MULTICHOICE for all other schoice items
+          if(x$metainfo$clozetype[i] == "mchoice") {
+	    cloze_mchoice_display_i <- if(is.null(cloze_mchoice_display)) "MULTIRESPONSE" else cloze_mchoice_display
+	    if(cloze_mchoice_display_i %in% moodle_schoice_display) {
+	      warning("MULTICHOICE-type displays should not be used for mchoice items, maybe it was intended to specify 'cloze_schoice_display'?")
+	    }
+	  } else {
+	    ## try to catch old-style cloze_mchoice_display for schoice elements
+	    if(x$metainfo$clozetype[i] == "schoice" &&
+	       is.null(cloze_schoice_display) &&
+	       !is.null(cloze_mchoice_display) &&
+	       cloze_mchoice_display %in% moodle_schoice_display) {
+	      warning("MULTICHOICE-type display was specified in 'cloze_mchoice_display' rather than 'cloze_schoice_display'")
+	      cloze_schoice_display <- cloze_mchoice_display
+	    }
+	    cloze_mchoice_display_i <- if(!is.null(cloze_schoice_display)) {
+	      cloze_schoice_display
+	    } else if(any(grepl("\\(", ql, fixed = TRUE) & grepl("\\)", ql, fixed = TRUE))) {
+	      "MULTICHOICE_V"
+	    } else {
+	      "MULTICHOICE"
+	    }
+	  }
+	  ## FIXME: Warn if the selected display option cannot work? (e.g., mchoice or math?)
+          tmp <- paste('{', points2[i], ':', cloze_mchoice_display_i, ':', sep = '')
+
+          ## set up Moodle percent fractions for correct and incorrect items
+	  ## -> use "=" instead of "%...%" for correct items if they sum up to 100%
           frac <- solution[[i]]
-          if(length(frac) < 2)
-            frac <- c(frac, !frac)
+          if(length(frac) < 2) frac <- c(frac, !frac)
           frac2 <- frac
-          pv <- eval$pointvec(frac)
+	  eval_i <- eval
+          if(!("pointvec" %in% names(eval_i))) {
+	    if(is.null(eval_i$rule)) eval_i$rule <- "none"
+            eval_i <- do.call("exams_eval", eval_i)
+          }
+          pv <- eval_i$pointvec(frac) ## FIXME: this passes correct as a logical rather as a character, is this intended?
           frac[frac2] <- pv["pos"]
           frac[!frac2] <- pv["neg"]
           p <- moodlePercent(frac)
+          if(isTRUE(all.equal(100, sum(as.numeric(p[frac2])), tol = 1e-5))) {
+	    p <- paste0("%", p, "%")
+            p[frac2] <- "="
+	    if(all(p[!frac2] == "%0%")) p[!frac2] <- ""
+	  } else {
+	    p <- paste0("%", p, "%")
+	  }
 
           if(k < 2) {
             tmp <- paste(ql, tmp)
-            p <- paste('%', p, '%', sep = '')
-            p[2] <- paste('~', p[2], sep = '')
-            ql <- paste(p, truefalse[rev(frac2 + 1)], sep = '', collapse = '')
+            p[2] <- paste0('~', p[2])
+            ql <- paste0(p, truefalse[rev(frac2 + 1)], collapse = '')
           } else {
             ql2 <- NULL
             for(j in 1:k)
-              ql2 <- paste(ql2, if(j > 1) '~' else NULL, paste('%',  p[j], '%', sep = ''), ql[j], sep = '')
+              ql2 <- paste0(ql2, if(j > 1) '~' else NULL, p[j], ql[j])
             ql <- ql2
           }
-          tmp <- paste(tmp, ql, sep = '')
-          tmp <- paste(tmp, '}', sep = '')
+          tmp <- paste0(tmp, ql)
+          tmp <- paste0(tmp, '}')
         }
         if(x$metainfo$clozetype[i] == "num") {
           for(j in 1:k) {
-            tmp <- c(tmp, paste(ql[j], ' {', points[i], ':NUMERICAL:=', solution[[i]][j],
-              ':', max(tol[[i]]), if(numwidth) paste('~%0%', fnums, ":0", sep = '') else NULL,
-              '}', sep = ''))
+            tmp <- c(tmp, paste0(ql[j], ' {', points2[i], ':NUMERICAL:=', solution[[i]][j],
+              ':', tol[[i]][j],
+	      if(!identical(numwidth, FALSE)) paste0('~%0%', fnum, ":0") else NULL,
+              '}'))
           }
         }
         if(x$metainfo$clozetype[i] == "string") {
           for(j in 1:k) {
-            tmp <- c(tmp, paste(ql[j], ' {', points[i], ':SHORTANSWER:%100%', solution[[i]][j],
-              if(!usecase) paste('~%100%', tolower(solution[[i]][j]), sep = '') else NULL,
-              '}', sep = ''))
+            tmp <- c(tmp, paste0(ql[j], ' {', points2[i], ':SHORTANSWER:%100%', gsub("}", "\\}", solution[[i]][j], fixed = TRUE),
+              if(!usecase && tolower(solution[[i]][j]) != solution[[i]][j]) paste0('~%100%', tolower(gsub("}", "\\}", solution[[i]][j], fixed = TRUE))) else NULL,
+	      if(!identical(stringwidth, FALSE)) paste0('~%0%', fstring) else NULL,
+              '}'))
           }
         }
         if(x$metainfo$clozetype[i] == "verbatim") {
           for(j in 1:k) {
-            tmp <- c(tmp, paste0(ql[j], ' {', points[i], solution[[i]][j], '}'))
+            tmp <- c(tmp, paste0(ql[j], ' {', points2[i], solution[[i]][j], '}'))
           }
         }
 
@@ -505,6 +651,12 @@ make_question_moodle23 <- function(name = NULL, solution = TRUE, shuffle = FALSE
       if(!is.null(qtext) & enumerate)
         qtext <- c('<ol type = "a">', paste('<li>', qtext, '</li>'), '</ol>')
       qtext <- c(x$question, qtext)
+      ## add abstention option (if any)
+#      if(abstention != "") {
+#        qtext <- c(qtext,
+#          paste0('<p>', abstention, ' {0:', cloze_mchoice_display_i, ':%100%', truefalse[1], '~%0%', truefalse[2], '} </p>')
+#        )
+#      }
       xml <- gsub('##QuestionText##', paste(qtext, collapse = "\n"), xml, fixed = TRUE)
     }
 
@@ -538,3 +690,14 @@ moodlePercent <- function(p)
   as.character(mp)
 }
 
+## Count decimal places.
+dc <- function(x) {
+  if((x %% 1) != 0) {
+    strs <- strsplit(as.character(format(x, scientific = F)), "\\.")
+    n <- nchar(strs[[1]][2])
+  } else {
+    n <- 0
+  }
+  return(n) 
+}
+
