@@ -8,16 +8,23 @@
   "jpeg","image/jpeg",
   "gif", "image/gif",
   "svg", "image/svg+xml",
+  "wmf", "application/x-msmetafile",
 
   "doc", "application/msword",
   "docx","application/vnd.openxmlformats-officedocument.wordprocessingml.document",
   "htm", "text/html",
   "html","text/html",
   "pdf", "application/pdf",
+  "rtf", "application/rtf",
   "tex", "application/x-tex",
   "txt", "text/plain",
   "xml", "application/xml",
 
+  "c",   "text/plain",
+  "py",  "text/plain",
+  "r",   "text/plain",
+  "rmd", "text/plain",
+
   "csv", "text/csv",
   "dta", "application/octet-stream",
   "ods", "application/vnd.oasis.opendocument.spreadsheet",
